searchData={"items":[{"type":"module","title":"WeatherflowTempest","doc":"This module is merely used as a convenience for accessing the non-OTP\nfunctions of the overall library.","ref":"WeatherflowTempest.html"},{"type":"function","title":"WeatherflowTempest.get_hub_serials/0","doc":"","ref":"WeatherflowTempest.html#get_hub_serials/0"},{"type":"function","title":"WeatherflowTempest.get_latest/0","doc":"","ref":"WeatherflowTempest.html#get_latest/0"},{"type":"function","title":"WeatherflowTempest.get_packet_stats/0","doc":"","ref":"WeatherflowTempest.html#get_packet_stats/0"},{"type":"function","title":"WeatherflowTempest.subscribe_to_udp_events/0","doc":"","ref":"WeatherflowTempest.html#subscribe_to_udp_events/0"},{"type":"module","title":"WeatherflowTempest.Client","doc":"Listens for packets from Weatherflow devices on the LAN, parses them, and\nstores their latest state/update in its own state, while emitting the\nparsed events via either a callback function or a Phoenix.PubSub broadcast.","ref":"WeatherflowTempest.Client.html"},{"type":"module","title":"Changes from UDP API - WeatherflowTempest.Client","doc":"It's important to note that we make some changes to the structure of the\nresults returned by the raw WeartherFlow UDP API:\n  * Events containing observations are flattened into a single observation\n  with all observation keys as top-level keys, rather than objects with\n  nested \"obs\" keys.\n  * Events containing lists of observations are emitted as multiple events. \n  * Event names are altered to be more descriptive. See the\n    [Event Examples](#module-event-examples) section below for event names\n    and example return data.\n\nIt is unclear when the devices will actually return a list of observations in\na single \"obs\" list, and in testing with live devices it hasn't actually been\nobserved.   \nHowever, to accommodate the case that it might happen what we'll do is create\na unique event for every item in the observation list, and emit them in order\nof ascending timestamp. These will be the flattened events described above.  \nThis simplifies handling the events from the perspective of users of the\nlibrary, since it makes the expected output completely predictable.","ref":"WeatherflowTempest.Client.html#module-changes-from-udp-api"},{"type":"module","title":"Usage - WeatherflowTempest.Client","doc":"There are two ways to get data from the Client: \n  1) via a callback function\n  2) via Phoenix.PubSub\n\nYou can use either, or both, depending on your needs.\n\n#","ref":"WeatherflowTempest.Client.html#module-usage"},{"type":"module","title":"via Callback Function - WeatherflowTempest.Client","doc":"A callback function is passed in to the `start_link` function as part of the\noptions list, under the \"callback_func\" key. (Multiple can be passed in, and\nwill all be called.)\nThe callback function will be called with two arguments:\n  1) the event type, as an atom\n  2) the event data, as a map\n\n```elixir\ndef handle_weatherflow_event(event_type, event_data) do\n  # do something with the data received from the event\nend\n\n{:ok, pid} = WeatherflowTempest.Client.start_link([callback_func: &handle_weatherflow_event/2])\n```\n\n> ##","ref":"WeatherflowTempest.Client.html#module-via-callback-function"},{"type":"module","title":"Blocking Callback Warning {: .warning} - WeatherflowTempest.Client","doc":"> Your callback functions will block the client from processing any further\n> data, so efforts should be made to keep them as lightweight as possible,\n> and hand off any complex processing.\n\nIt is worth noting that the callback function will *not* be notified of any\nJSON parsing errors, only successfully parsed events.\n\n#","ref":"WeatherflowTempest.Client.html#module-blocking-callback-warning-warning"},{"type":"module","title":"via Phoenix.Pubsub - WeatherflowTempest.Client","doc":"If you'd prefer to receive events via Phoenix.PubSub, you can configure the\npubsub you'd like to use in your config file:\n```elixir\nconfig :weatherflow_tempest, :pubsub_name, MyApp.PubSub\n```\n\nYou must also start the WeatherflowTempest.Client, commonly as a child of\nyour Application Supervisor in application.ex:\n```elixir\ndef start(_type, _args) do\n  children = [\n    {WeatherflowTempest.Client, []},\n  ]\n  opts = [strategy: one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nEvents are published as an {{:weatherflow, event_type}, event_data} struct,\non the topic \"weatherflow:udp\".\n\nTo subscribe to all events the convenience function\n`WeatherflowTempest.PubSub.subscribe_to_udp_events/0` is provided.\nHandling all broadcast weatherflow events looks something like:\n```elixir\ndef handle_info({{:weatherflow, event_type}, event_data}, socket) do\n  IO.puts(\"Got a #{event_type} message!\")\n  {:noreply, socket}\nend\n```\nAnd of course you can match on specific event types as well.\n\nEvents are emitted to the PubSub in ascending timestamp order, but due to\nthe nature of PubSub if your usage requires that events be processed in\nstrict timeline order you may wish to use the callback function instead.","ref":"WeatherflowTempest.Client.html#module-via-phoenix-pubsub"},{"type":"module","title":"Event Examples - WeatherflowTempest.Client","doc":"Both methods of receiving events will receive the same event_types and the\nsame event_data objects, the only difference is how they are received by your\napplication.\n\nAny given \"hub\" should only emit both :observation_air and :observation_sky\nevents, or only emit :observation_tempest events, depending on which product\nit is. Keeping all three observation types for each hub allows the calling\napplication to easily handle both types of hubs simply by matching on the\nevent type or value in the `%WeatherflowTempest.Client.Hub{}` struct. \n(The alternative, having a single :observation event/field, would require that\nwe embed the product type in the data, which would be a bit less clear to\nhandle since each type of observation contains different fields.)\n\nNote that we make no effort to coerce any data types from the API, since\nthe UDP API does not document it's data types. The examples shown below\nuse the same data as the examples in the Weatherflow API docs for each\nevent type. We assume that the values shown in their examples are\nstandardized, and if the API returns a float they would have shown 0.0.\nWho knows though. So handle with care.\n\nExamples of the map returned by all event types are documented below:\n\n#### :event_precipitation\n```elixir\n%{\n  serial_number: \"SK-00008453\",\n  hub_sn: \"HB-00000001\",\n  timestamp: ~U[2017-04-27 19:47:25Z]\n}\n```\n\n#### :event_strike\n```elixir\n%{\n  serial_number: \"AR-00004049\",\n  hub_sn: \"HB-00000001\",\n  timestamp: ~U[2017-04-27 19:47:25Z]\n}\n```\n\n#### :rapid_wind\n```elixir\n%{\n  serial_number: \"SK-00008453\",\n  hub_sn: \"HB-00000001\",\n  timestamp: ~U[2017-04-27 19:47:25Z],\n  wind_speed_mps: 2.3,\n  wind_direction_degrees: 128\n}\n```\n\n#### :observation_air\n```elixir\n%{\n  serial_number: \"AR-00004049\",\n  hub_sn: \"HB-00000001\",\n  firmware_revision: 17,\n  timestamp: ~U[2017-04-26 00:00:35Z],\n  station_pressure_MB: 835.0,\n  air_temperature_C: 10.0,\n  relative_humidity_percent: 45,\n  lightningstrike_count: 0,\n  lightningstrike_avg_distance_km: 0,\n  battery: 3.46,\n  reportinterval_mintues: 1\n}\n```\n\n#### :observation_sky\n```elixir\n%{\n  serial_number: \"SK-00008453\",\n  hub_sn: \"HB-00000001\",\n  firmware_revision: 29,\n  timestamp: ~U[2017-04-27 19:29:00Z],\n  illuminance_lux: 9000,\n  uv_index: 10,\n  rain_accumulated_mm: 0.0,\n  wind_lull_ms: 2.6,\n  wind_avg_ms: 4.6,\n  wind_gust_ms: 7.4,\n  wind_direction_degrees: 187,\n  battery_volts: 3.12,\n  reportinterval_minutes: 1,\n  solar_radiation_wm2: 130,\n  local_day_rain_accumulation: nil,\n  precipitation_type: :none,\n  wind_sample_interval_seconds: 3\n}\n```\nNote that the \"precipitation_type\" field is parsed to a human-readable format\nrather than the strict integer format used by the API. Values returned will\nbe one of :none, :rain, :hail, or :rain_plus_hail\n\n#### :observation_tempest\n```elixir\n%{\n  serial_number: \"ST-00000512\",\n  hub_sn: \"HB-00013030\",\n  firmware_revision: 129,\n  timestamp: ~U[2020-05-08 14:36:54Z],\n  wind_lull_ms: 0.18,\n  wind_avg_ms: 0.22,\n  wind_gust_ms: 0.27,\n  wind_direction_degrees: 144,\n  wind_sample_interval_seconds: 6,\n  station_pressure_MB: 1017.57,\n  air_temperature_C: 22.37,\n  relative_humidity_percent: 50.26,\n  illuminance_lux: 328,\n  uv_index: 0.03,\n  solar_radiation_wm2: 3,\n  precip_accumulated_mm: 0.000000,\n  precipitation_type: :none,\n  lightningstrike_avg_distance_km: 0,\n  lightningstrike_count: 0,\n  battery_volts: 2.410,\n  reportinterval_minutes: 1\n}\n```\nNote that the \"precipitation_type\" field is parsed to a human-readable format\nrather than the strict integer format used by the API. Values returned will\nbe one of :none, :rain, :hail, or :rain_plus_hail\n\n#### :device_status\n```elixir\n%{\n  serial_number: \"AR-00004049\",\n  hub_sn: \"HB-00000001\",\n  timestamp: ~U[2017-11-16 18:12:03Z],\n  uptime: 2189,\n  uptime_string: \"36 minutes, 29 seconds\",\n  voltage: 3.50,\n  firmware_revision: 17, \n  rssi: -17,\n  hub_rssi: -87,\n  sensor_status: %{\n    sensors_okay: true,\n    lightning_failed: false,\n    lightning_noise: false,\n    lightning_disturber: false,\n    pressure_failed: false,\n    temperature_failed: false,\n    rh_failed: false,\n    wind_failed: false,\n    precip_failed: false,\n    light_uv_failed: false,\n    power_booster_depleted: false,\n    power_booster_shore_power: false,\n  },\n  debug: false,\n}\n```\n\n#### :hub_status\n```elixir\n%{\n  hub_sn: \"HB-00000001\",\n  serial_number: \"HB-00000001\",\n  firmware_revision: \"35\",\n  uptime: 1670133,\n  uptime_string: \"2 weeks, 5 days, 7 hours, 55 minutes, 33 seconds\",\n  rssi: -62,\n  timestamp: ~U[2017-05-25 15:04:51Z],\n  reset_flags: [\"Brownout reset\", \"PIN reset\", \"Power reset\"],\n  seq: 48,\n  fs: :not_parsed__internal_use_only,\n  radio_stats: %{\n    version: 2,\n    reboot_count: 1,\n    i2c_bus_error_count: 0,\n    radio_status: \"Radio Active\",\n    radio_network_id: 2839\n  },\n  mqtt_stats: :not_parsed__internal_use_only,\n}\n```","ref":"WeatherflowTempest.Client.html#module-event-examples"},{"type":"function","title":"WeatherflowTempest.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"WeatherflowTempest.Client.html#child_spec/1"},{"type":"function","title":"WeatherflowTempest.Client.get_hub_serials/0","doc":"Get a list of serial numbers of Weatherflow Hubs that have been heard from.","ref":"WeatherflowTempest.Client.html#get_hub_serials/0"},{"type":"function","title":"Examples - WeatherflowTempest.Client.get_hub_serials/0","doc":"iex> WeatherflowTempest.Client.get_hub_serials()\n    [\"HUB_SERIAL_ONE\", \"HUB_SERIAL_TWO\"]","ref":"WeatherflowTempest.Client.html#get_hub_serials/0-examples"},{"type":"function","title":"WeatherflowTempest.Client.get_latest/0","doc":"Get all the latest data that the client has heard.\n\nNote that the resulting WeatherflowTempest.Client.Hub struct may contain\nempty fields if the client hasn't heard certain types of events yet.","ref":"WeatherflowTempest.Client.html#get_latest/0"},{"type":"function","title":"Examples - WeatherflowTempest.Client.get_latest/0","doc":"iex> WeatherflowTempest.Client.get_latest()\n    %{\n      \"HUB_SERIAL_ONE\" => %WeatherflowTempest.Client.Hub{...}\n    }","ref":"WeatherflowTempest.Client.html#get_latest/0-examples"},{"type":"function","title":"WeatherflowTempest.Client.get_packet_stats/0","doc":"Get the total number of UDP packets and errors received by the client.","ref":"WeatherflowTempest.Client.html#get_packet_stats/0"},{"type":"function","title":"Examples - WeatherflowTempest.Client.get_packet_stats/0","doc":"iex> WeatherflowTempest.Client.get_packet_stats()\n    %{\n      packets_parsed: 123,\n      packet_errors: 0\n    }","ref":"WeatherflowTempest.Client.html#get_packet_stats/0-examples"},{"type":"module","title":"WeatherflowTempest.Client.Hub","doc":"A struct representing the most recent data and status of a particular hub.\n\nNote that if a particular observation type hasn't yet been heard for this\nhub then some keys may be empty maps.","ref":"WeatherflowTempest.Client.Hub.html"},{"type":"type","title":"WeatherflowTempest.Client.Hub.t/0","doc":"","ref":"WeatherflowTempest.Client.Hub.html#t:t/0"},{"type":"module","title":"WeatherflowTempest.Protocol","doc":"The Weatherflow Protocol has a lot of magic fields. This parses and converts\nthem to make the returned objects more intelligible.\n\nByte-effecient arrays are unpacked into named fields based on the protocol docs\npublished by Weatherflow.\n\nThe following field standardizations are made to all event types:\n  * \"type\" fields are removed. (Use the whole `{:type, %{}}` tuple)\n  * \"evt\" fields containing the raw un-parsed event data are removed, and the data\n    from those fields is flattened into a single event map.\n  * \"uptime\" fields containing seconds-as-integers are given human-readable string\n    representations such as \"1 week, 4 days, 3 hours, 16 minutes\", which are\n    placed in an :uptime_string field\n  * \"timestamp\" field containing the epoch time are converted to DateTime\n  * All fields are converted to use atoms as keys. The only string keys used will\n    be device serial numbers, i.e.:\n\n    ```\n      device_statuses: %{\n          \"AR-00000001\" => %{\n              firmware_revision: 23\n          }\n      }\n    ```\n\n  * :hub_sn key is added to the hub_status message type, allowing easy pattern\n    matching for all events from a given hub.","ref":"WeatherflowTempest.Protocol.html"},{"type":"module","title":"Notes: - WeatherflowTempest.Protocol","doc":"* The hub_status event returns a firmware_revision as a string rather than an\n  integer, according to the API examples. We do **not** convert that integer to\n  a string since it's unclear what Weatherflow's intent here is. Perhaps there\n  will be a firmware revision \"37beta2\"?\n* The actual data types of the fields is not given by the Weatherflow UDP API\n  documentation. We infer which values are floats and which are ints based on\n  their examples only. There is no guarantee that this is correct. Did they\n  show \"0\" for an example but may actually return \"0.1\"? Because of this we\n  will not attempt to coerce any values or parse anything that we aren't\n  absolutely certain of.","ref":"WeatherflowTempest.Protocol.html#module-notes"},{"type":"function","title":"WeatherflowTempest.Protocol.handle_json/1","doc":"Accepts the result tuple of Jason.decode()\nIf the JSON could not be decoded bubble the error up to be handled by the\n`WeatherflowTempest.Client`, otherwise parse the event types defined by\nthe Weatherflow spec.\n\nReturns a tuple containing an atom matching the event \"type\" field, followed\nby the parsed object as a map. (Because it is cleaner to pattern match the\nagainst the tuple than against a key in the map, and makes returning ar error\ntuple the easy way to bubble up any errors.)\nSome liberties are taken with renaming fields and altering the resulting\nstructure for clarity. The \"type\" key is removed from all result maps, as the\nintention is to hold on to the entire tuple if you need to pass it around.\n\nFor evt_precip, evt_strike, and rapid_wind messages we flatten the data into\na single map for convenience. \n\nFor example, a rapid_wind event from the UDP API looks like this:\n```json\n  {\n    \"serial_number\": \"AR-00004049\",\n    \"type\":\"rapid_wind\",\n    \"hub_sn\": \"HB-00000001\",\n    \"ob\":[1493322445,2.3,128]\n  }\n```\nand we will parse that into:\n```elixir\n  {:rapid_wind,\n   %{\n    serial_number: \"AR-00004049\",\n    hub_sn: \"HB-00000001\",\n    timestamp: ~U[2017-04-27 19:47:25Z],\n    wind_speed_mps: 2.3,\n    wind_direction_degrees: 128\n   }\n  }\n```\n\nHowever, the obs_air, obs_sky, and obs_st event types can return a list of\nobservations, although they most often return just a single event.\nBecause of this those observations are **not** flattened into a single map,\nand are instead returned as a list, sorted by ascending timestamp.\nThey are returned under the :observations key, rather than the \"obs\" used\nby the API, for the sake of clarity.\n\nThe API doesn't cearly define what would cause the devices to return a list\nof observations rather than a single one, so we can't make any assumptions\nhere about de-duplication or repeated events unfortunately, and will simply\nparse and return whatever the API provides. Do note however that the logic\nin `WeatherflowTempest.Client` strips out all but the most recent event,\nso if you'd like to actually deal with a device that is sending lists of\nobservations you'll need to use `WeatherflowTempest.Protocol` directly.","ref":"WeatherflowTempest.Protocol.html#handle_json/1"},{"type":"module","title":"WeatherflowTempest.PubSub","doc":"Publishes events via Phoenix.PubSub.\n\nIn order to use Phoenix.PubSub broadcasting you must ensure that your\napplication also require phoenix_pubsub in its deps.\n\nConfigure the pubsub you'd like to use in the appropriate config file:\n```elixir\nconfig :weatherflow_tempest, :pubsub_name, MyApp.PubSub\n```\n\nEvents are published as an {{:weatherflow, event_type}, event_data} struct,\non the \"weatherflow:udp\" topic. \nIf no pubsub_name is defined in the config, then no PubSub messages are\nbroadcast.\n\nThe tuple {:weatherflow, event} is used as they key in order to make it easy\nto match against all Weatherflow related events if your pubsub subscriber\nalso receives messages from other pubsub topics.\n\nThe `event` is an expanded version of the event type from the weatherflow API,\nand the parsed object as the payload. \nFull documentation for the event names and structures is available in the\n`WeatherflowTempest.Client` module.","ref":"WeatherflowTempest.PubSub.html"},{"type":"function","title":"WeatherflowTempest.PubSub.get_pubsub_name/0","doc":"Return the name of the pubsub we're using.\nMight be useful if you want to subscribe to the UDP events yourself, or\ncheck on which pubsub is being used at runtime","ref":"WeatherflowTempest.PubSub.html#get_pubsub_name/0"},{"type":"function","title":"WeatherflowTempest.PubSub.subscribe_to_udp_events/0","doc":"Convenience method to subscribe to the correct pubsub name and channel to\nreceive all parsed UDP events received as pubsub messages.","ref":"WeatherflowTempest.PubSub.html#subscribe_to_udp_events/0"},{"type":"extras","title":"Weatherflow Tempest","doc":"# Weatherflow Tempest\n\n> A library for handling the data from the LAN API for WeatherFlow weather stations.\n\nOnline docs found at [https://hexdocs.pm/weatherflow_tempest](https://hexdocs.pm/weatherflow_tempest).  \nCode and bug reports are at [https://github.com/joshproehl/weatherflow_tempest](https://github.com/joshproehl/weatherflow_tempest).\n\nCurrent Weatherflow UDP API version targeted is [171](https://weatherflow.github.io/Tempest/api/udp/v171/).\n\nSupported devices:\n- Air/Sky\n- Tempest\n\nYes, the library is called \"weatherflow tempest\". Why it was created that way,\nwhen it has always supported both Tempest and the earlier device, is lost to\ntime. Maybe because it was going to handle a veritable tempest of UDP packets?","ref":"readme.html"},{"type":"extras","title":"Installation - Weatherflow Tempest","doc":"Add `weatherflow_tempest` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_pubsub, \"~> 2.0\"}, # Only required if using PubSub broadcasts\n    {:weatherflow_tempest, \"~> 1.0.0\"}\n  ]\nend\n```\nand fetch your dependencies with\n```\n$ mix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Weatherflow Tempest","doc":"The intended way to use the library is to configure it to use a Phoenix.PubSub\ninstance, let it auto-start the client, and then subscribe to the data via\nPhoenix.PubSub.\n\n```elixir\n# In your config file:\n\nconfig :weatherflow_tempest, pubsub_name: MyApp.PubSub\n```\n\n```elixir\n######################################\n# Example PubSub usage in a LiveView #\n\ndef mount(_params, _session, socket) do\n  WeatherflowTempest.PubSub.subscribe_to_udp_events()\n  {:ok, socket}\nend\n\ndef handle_info({{:weatherflow, event_type}, event_data}, socket) do\n  IO.puts(\"Got a \\#\\{event_type\\} message!\")\n  # Update your LiveView socket with the data!\n  {:noreply, socket}\nend\n```\n\nThe choice to use Phoenix.PubSub was made because re-implementing PubSub to\nthe same level would be foolish, and also the primary use of this library is\nvery likely to display the data, probably in something like a Phoenix\nLiveView.\n\nHowever, if for some reason you need more direct usage of the data, using\ncallback functions is also supported. To do this you must configure the\nlibrary not to auto-start the Client, and then manually start and supervise\nthe Client.\n\n```elixir\n# In your config file:\n\nconfig :weatherflow_tempest, callbacks_only: true\n```\n\n```elixir\n#######################################\n# Example usage via Callback function #\n\ndef handle_weatherflow_event(event_type, event_data) do\n  # do something with the data received from the event\nend\n\n{:ok, pid} = WeatherflowTempest.Client.start_link([callback_func: &handle_weatherflow_event/2])\n```\n\nFull configuration and usage for both methods can be found in the\n`WeatherflowTempest.Client` docs.\n\nYou can get the latest data heard from the station via\n`WeatherflowTempest.get_latest/0`, which will return a map containing the most\nrecent data that has been heard from every Weatherflow device on the LAN, but\nin practice you will probably want to use either the PubSub broadcasts or \ncallback functions to handle events as they occur, and only use\n`WeatherflowTempest.get_latest/0` to seed initial values to your app. (When\nstarting a new LiveView page for example.)\n\n> ##","ref":"readme.html#usage"},{"type":"extras","title":"Note {: .info} - Weatherflow Tempest","doc":"> \n> While it is technically possible to start the application with neither a\n> PubSub name configured nor a callback function given, it is not the intended\n> way to use the library.  \n> In this case the Client will still start, but you will only be able to access\n> the data via the `WeatherflowTempest.get_latest/0` function, and will not be\n> able to receive individual events.\n\nThe payload for each event is transformed from the UDP API strucure by\n`WeatherFlowTempest.Client`, but in short it will be a map containing all data returned\nby the API as key/value pairs. Examples of returned data can be found in \n[Event Examples](WeatherflowTempest.Client.html#module-event-examples).","ref":"readme.html#note-info"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.0.1 - Changelog","doc":"* Documentation updates, including fixing incorrect examples\n\n#","ref":"changelog.html#1-0-1"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Changed rain accumulation field in Sky observations to include units.\n  local_day_rain_accumulation became local_day_rain_accumulation_mm\n* Changed battery field in Air observations to include units. \n  It is now battery_volts, like the other device types use.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"1.0.0 - Changelog","doc":"* Update Elixir dependency to 1.10 or greater\n* Documentation and code formatting updates\n* Update for changes in Weatherflow API v171\n\n#","ref":"changelog.html#1-0-0"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added :hub_sn key to the hub_status event. (Duplicates the :serial_number key\n  as a convenience to allow matching against :hub_sn on all event types.)\n* Added ability to pass callback functions to the client that will be used\n  to handle parsed weatherflow events. (No longer strictly depends on pubsub)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"Version 1.0.0 has changed the API and will very likely break *any* existing uses\nof the library. (This is why it's being released as 1.0.0, rather than 0.2.0)\n\n  * The structs returned by library have changed. Keys are standardized to use\n    atoms for all protocol keys, and only use strings for device serial numbers.\n    For example:\n\n    ```elixir\n    device_statuses: %{\n        \"AR-00000001\" => %{\n            firmware_revision: 23\n        }\n    }\n    ```\n\n  * Returned :uptime key to being integer second (String description that was\n    previously here is now in the :uptime_string field.)\n  * Renamed WeatherflowTempest.PubSub.subscribe_udp_events/0 to\n    `WeatherflowTempest.PubSub.subscribe_to_udp_events/0` for linguistic clarity.\n  * Renamed the :windspeed_mps in a rapid_wind event to :wind_speed_mps for\n    consistency with other field names.\n  * Events are now emitted via the Phoenix.PubSub.broadcast method as\n    {{:weatherflow, event_name}, object} tuples, rather than using\n    %Phoenix.Socket.Broadcast{} struct, allowing the library to work correctly\n    outside of a Phoenix app.\n  * No longer automatically starts a Phoenix.PubSub if one is not defined\n    via config.\n  * No longer has Phoenix.PubSub as a :prod dependency, must be required by the\n    parent app if the PubSub is going to be used.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"0.1.0 - Changelog","doc":"* Initial Release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2021 Josh Proehl\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}